ifndef QCONFIG
QCONFIG=qconfig.mk
endif
include $(QCONFIG)

#where to install vsomeip:
#$(INSTALL_ROOT_$(OS)) is pointing to $QNX_TARGET
#by default, unless it was manually re-routed to
#a staging area by setting both INSTALL_ROOT_nto
#and USE_INSTALL_ROOT
VSOMEIP_INSTALL_ROOT ?= $(INSTALL_ROOT_$(OS))

#where to find the vsomeip external dependencies, such as Boost
VSOMEIP_EXTERNAL_DEPS_INSTALL ?= $(USE_ROOT_$(OS))

#choose Release or Debug
CMAKE_BUILD_TYPE ?= Release

#set the following to TRUE if you want to compile the vsomeip tests.
#If you do, make sure to set GTEST_ROOT to point to the google test library sources
GENERATE_TESTS ?= TRUE
TEST_IP_MASTER ?= 172.31.1.3
TEST_IP_SLAVE ?= 172.31.1.2

#set the following to FALSE if generating .pinfo files is causing problems
GENERATE_PINFO_FILES ?= TRUE

#override 'all' target to bypass the default QNX build system
ALL_DEPENDENCIES = vsomeip_all
.PHONY: vsomeip_all

FLAGS   += -g -D_QNX_SOURCE
LDFLAGS += -Wl,--build-id=md5 -lang-c++ -lsocket
ifeq (qnx6,$(notdir $(QNX_TARGET:%/=%)))
FLAGS   += -D_GLIBCXX_USE_C99 -Y _gpp
LDFLAGS += -Y _gpp
CXX_LIBTYPE = GNU
GCC_VER = "8.3.0"
else
CXX_LIBTYPE = LLVM
GCC_VER = ""
endif

CMAKE_ARGS = -DCMAKE_TOOLCHAIN_FILE=qnx.nto.toolchain.cmake \
             -DCMAKE_INSTALL_PREFIX=$(QNX_TARGET) \
             -DVSOMEIP_EXTERNAL_DEPS_INSTALL=$(VSOMEIP_EXTERNAL_DEPS_INSTALL) \
             -DCMAKE_BUILD_TYPE=$(CMAKE_BUILD_TYPE) \
             -DCMAKE_MODULE_PATH=$(PROJECT_ROOT) \
             -DEXTRA_CMAKE_C_FLAGS="$(FLAGS)" \
             -DEXTRA_CMAKE_CXX_FLAGS="$(FLAGS)" \
             -DEXTRA_CMAKE_LINKER_FLAGS="$(LDFLAGS)" \
             -DCPUVARDIR=$(CPUVARDIR) \
             -DGENERATE_PINFO_FILES=$(GENERATE_PINFO_FILES) \
             -DCXX_LIBTYPE=${CXX_LIBTYPE} \
             -DGCC_VER=${GCC_VER} \
			 -DVSOMEIP_INSTALL_ROUTINGMANAGERD=ON \
             -DENABLE_COMPAT=ON

ifeq ($(GENERATE_TESTS), TRUE)
CMAKE_ARGS += -DGENERATE_TESTS=1 \
              -DENABLE_SIGNAL_HANDLING=1 \
              -DTEST_IP_MASTER=$(TEST_IP_MASTER) \
              -DTEST_IP_SLAVE=$(TEST_IP_SLAVE)
endif

MAKE_ARGS ?= -j $(firstword $(JLEVEL) 4)

define PINFO
endef
PINFO_STATE=Experimental
USEFILE=

include $(MKFILES_ROOT)/qtargets.mk

ifneq ($(wildcard $(foreach dir,$(LIBVPATH),$(dir)/libregex.so)),)
	LDFLAGS += -lregex
endif


ifndef NO_TARGET_OVERRIDE
vsomeip_all:
	@mkdir -p build
	cd build && cmake $(CMAKE_ARGS) ../../../../
	cd build && make all $(MAKE_ARGS)

check: vsomeip_all
	@cd build && make build_tests gen_testsInfo install $(MAKE_ARGS)

install: vsomeip_all
	@cd build && make install $(MAKE_ARGS)

clean iclean spotless:
	rm -fr build

cuninstall uninstall:

.PHONY: qnx_raw_vm
qnx_raw_vm:
	@../../../qnxhelper/basic-qvm.sh
endif

#everything down below deals with the generation of the PINFO
#information for shared objects that is used by the QNX build
#infrastructure to embed metadata in the .so files, for example
#data and time, version number, description, etc. Metadata can
#be retrieved on the target by typing 'use -i <path to vsomeip .so file>'.
#this is optional: setting GENERATE_PINFO_FILES to FALSE will disable
#the insertion of metadata in .so files.
ifeq ($(GENERATE_PINFO_FILES), TRUE)
#the following rules are called by the cmake generated makefiles,
#in order to generate the .pinfo files for the shared libraries
%.so.3.1.20.3:
	$(ADD_PINFO)
	$(ADD_USAGE)

%vsomeipd:
	$(ADD_PINFO)
	$(ADD_USAGE)
endif
