/* Copyright Siemens, 2023 */

#include "../include/serializer.hpp"
#include "../include/deserializer.hpp"
#include "../include/message_base_impl.hpp"
#include "../include/someip_message_header_impl.hpp"

namespace vsomeip_v3 {

bool someip_message_header_impl::serialize(serializer *_to) const {
    return (0 != _to
            && _to->serialize(service_)
            && _to->serialize(method_)
            && _to->serialize(owner_->get_length())
            && _to->serialize(client_)
            && _to->serialize(session_)
            && _to->serialize(protocol_version_)
            && _to->serialize(interface_version_)
            && _to->serialize(static_cast<uint8_t>(type_))
            && _to->serialize(static_cast<uint8_t>(code_)));
}

bool someip_message_header_impl::deserialize(deserializer *_from) {
    bool is_successful;

    uint8_t tmp_message_type, tmp_return_code;

    is_successful = (0 != _from
            && _from->deserialize(service_)
            && _from->deserialize(method_)
            && _from->deserialize(length_)
            && _from->deserialize(client_)
            && _from->deserialize(session_)
            && _from->deserialize(protocol_version_)
            && _from->deserialize(interface_version_)
            && _from->deserialize(tmp_message_type)
            && _from->deserialize(tmp_return_code));

    if (is_successful) {
        type_ = static_cast< message_type_e >(tmp_message_type);
        code_ = static_cast< return_code_e >(tmp_return_code);
    }

    return is_successful;
}

} // namespace vsomeip_v3
