#!/bin/sh
# Copyright (C) 2015-2017 Bayerische Motoren Werke Aktiengesellschaft (BMW AG)
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

# Copyright Siemens, 2023

# Purpose: This script is needed to start the services with
# one command. This is necessary as ctest - which is used to run the
# tests - isn't able to start multiple binaries for one testcase. Therefore
# the testcase simply executes this script. This script then runs the services
# and checks that all exit successfully.

if [ $# -lt 1 ]
then
    echo "Please pass a json file to this script."
    echo "For example: $0 client_id_test_diff_client_ids_diff_ports_master.json"
    exit 1
fi

MASTER_JSON_FILE=$1
CLIENT_JSON_FILE=$(echo "$MASTER_JSON_FILE" | sed 's/master/slave/g')

FAIL=0

# Start the services
export VSOMEIP_APPLICATION_NAME=client_id_test_service_one
export VSOMEIP_CONFIGURATION=$1
./client_id_test_service 1 &
CLIENT_ID_PIDS[1]=$!

export VSOMEIP_APPLICATION_NAME=client_id_test_service_two
export VSOMEIP_CONFIGURATION=$1
./client_id_test_service 2 &
CLIENT_ID_PIDS[2]=$!

export VSOMEIP_APPLICATION_NAME=client_id_test_service_three
export VSOMEIP_CONFIGURATION=$1
./client_id_test_service 3 &
CLIENT_ID_PIDS[3]=$!

sleep 1

ssh -o "StrictHostKeyChecking=no" root@172.31.1.2 "cd /data/home/qnxuser/build/test && ./client_id_test_slave_starter_qnx.sh $CLIENT_JSON_FILE" &

# Wait until client and service are finished
for client_pid in "${CLIENT_ID_PIDS[@]}"
do
    if [ -n "$client_pid" ]; then
        # Fail gets incremented if either client or service exit
        # with a non-zero exit code
        wait "$client_pid" || ((FAIL+=1))
    fi
done

# Check if both exited successfully 
if [ $FAIL -eq 0 ]
then
    exit 0
else
    exit 1
fi
