#!/bin/sh
# Copyright (C) 2015-2017 Bayerische Motoren Werke Aktiengesellschaft (BMW AG)
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

# Copyright Siemens, 2023

# Purpose: This script is needed to start the client and service with
# one command. This is necessary as ctest - which is used to run the
# tests - isn't able to start two binaries for one testcase. Therefore
# the testcase simply executes this script. This script then runs client
# and service and checks that both exit successfully.

# Display a message to show the user that he must now call the external service
# to finish the test successfully

FAIL=0

ssh -o "StrictHostKeyChecking=no" root@172.31.1.2 "cd /data/home/qnxuser/build/test && ./magic_cookies_test_client_start_qnx.sh" &

# Start the client for magic-cookies test
export VSOMEIP_APPLICATION_NAME=magic_cookies_test_service
export VSOMEIP_CONFIGURATION=magic_cookies_test_service.json
./magic_cookies_test_service  --tcp --static-routing &

# Wait until client and service are finished
for job in $(jobs -p)
do
    # Fail gets incremented if either client or service exit
    # with a non-zero exit code
    wait $job || ((FAIL+=1))
done

# Check if client and server both exited successfully
if [ $FAIL -eq 0 ]
then
    exit 0
else
    exit 1
fi
