#!/bin/sh
# Copyright (C) 2015-2017 Bayerische Motoren Werke Aktiengesellschaft (BMW AG)
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

# Copyright Siemens, 2023

# Purpose: This script is needed to start the services with
# one command. This is necessary as ctest - which is used to run the
# tests - isn't able to start multiple binaries for one testcase. Therefore
# the testcase simply executes this script. This script then runs the services
# and checks that all exit successfully.

FAIL=0

if [ $# -lt 1 ]
then
    echo "Please pass a test mode to this script."
    echo "For example: $0 MALICIOUS_EVENTS"
    echo "Valid subscription types include:"
    echo "            [MALICIOUS_EVENTS, PROTOCOL_VERSION, MESSAGE_TYPE, RETURN_CODE, WRONG_HEADER_FIELDS_UDP]"
    exit 1
fi
TESTMODE=$1

export VSOMEIP_CONFIGURATION=malicious_data_test_master.json
# start daemon
../examples/routingmanagerd/./routingmanagerd &
PID_VSOMEIPD=$!
# Start the services
./malicious_data_test_service $TESTMODE &
PID_SERIVCE=$!

sleep 1

if [ ! -z "$USE_LXC_TEST" ]; then
    echo "Waiting for 5s"
    sleep 5
    echo "starting offer test on slave LXC offer_test_external_slave_starter.sh"
    ssh -tt -i $SANDBOX_ROOT_DIR/commonapi_main/lxc-config/.ssh/mgc_lxc/rsa_key_file.pub -o StrictHostKeyChecking=no root@$LXC_TEST_SLAVE_IP "bash -ci \"set -m; cd \\\$SANDBOX_TARGET_DIR/vsomeip_lib/test; ./malicious_data_test_msg_sender 172.31.1.3 172.31.1.2 $TESTMODE\"" &
    echo "remote ssh pid: $!"
elif [ ! -z "$USE_DOCKER" ]; then
    echo "Waiting for 5s"
    sleep 5
    docker exec $DOCKER_IMAGE sh -c "cd $DOCKER_TESTS && sleep 10; ./malicious_data_test_msg_sender 172.31.1.3 172.31.1.2 $TESTMODE" &
else
    ssh -o "StrictHostKeyChecking=no" root@172.31.1.2 "cd /data/home/qnxuser/build/test && sleep 10; ./malicious_data_test_msg_sender 172.31.1.3 172.31.1.2 $TESTMODE" &
    REMOTE_SCRIPT_PID=$!
fi

# Wait until all clients and services are finished
echo "waiting for $PID_SERIVCE"
wait $PID_SERIVCE
exit_status=$?
if [ $exit_status != 0 ] && [ $exit_status != 127 ]; then
    echo "Fail status $exit_status"
    FAIL=$FAIL+1
fi

# kill the services
kill $PID_VSOMEIPD
wait $PID_VSOMEIPD
exit_status=$?
if [ $exit_status != 0 ] && [ $exit_status != 127 ]; then
    echo "Fail status $exit_status"
    FAIL=$FAIL+1
fi
sleep 1

# Wait for the remote script to finish
wait $REMOTE_SCRIPT_PID
exit_status=$?
if [ $exit_status != 0 ] && [ $exit_status != 127 ]; then
    echo "Fail status $exit_status"
    FAIL=$FAIL+1
fi
sleep 1

# Check if everything went well
exit $FAIL
