#!/bin/sh
# Copyright (C) 2015-2017 Bayerische Motoren Werke Aktiengesellschaft (BMW AG)
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

# Copyright Siemens, 2023

# Purpose: This script is needed to start the client and service with
# one command. This is necessary as ctest - which is used to run the
# tests - isn't able to start two binaries for one testcase. Therefore
# the testcase simply executes this script. This script then runs client
# and service and checks that both exit sucessfully.

FAIL=0

# Parameter 1: the pid to check
check_tcp_udp_sockets_are_open ()
{
    # Check that the service does listen on at least one TCP/UDP socket 
    # awk is used to avoid the case when a inode number is the same as a PID. The awk
    # program filters the sockstat output down to the protocol (5th field) and 
    # the PID/Program name (3rd field) fields.
    SERVICE_SOCKETS_LISTENING=$(sockstat -46 2> /dev/null | awk '{print $3 "\t"  $5}' | grep $1 | wc -l)
    if [ $SERVICE_SOCKETS_LISTENING -lt 1 ]
    then
        ((FAIL+=1))
    fi
}

# Parameter 1: the pid to check
check_tcp_udp_sockets_are_closed ()
{
    # Check that the service does not listen on any TCP/UDP socket 
    # or has any active connection via a TCP/UDP socket
    # awk is used to avoid the case when a inode number is the same as a PID. The awk
    # program filters the sockstat output down to the protocol (5th field) and 
    # the PID/Program name (3rd field) fields.
    SERVICE_SOCKETS_LISTENING=$(sockstat -46 2> /dev/null | awk '{print $3 "\t"  $5}' | grep $1 | wc -l)
    if [ $SERVICE_SOCKETS_LISTENING -ne 0 ]
    then
        ((FAIL+=1))
    fi
}

# Start the service
export VSOMEIP_APPLICATION_NAME=external_local_routing_test_service
export VSOMEIP_CONFIGURATION=external_local_routing_test_service.json
./external_local_routing_test_service &
SERIVCE_PID=$!
sleep 2

check_tcp_udp_sockets_are_open $SERIVCE_PID

# Start the client (we reuse the one from the local_routing_test to check
# the local routing functionality).
export VSOMEIP_APPLICATION_NAME=local_routing_test_client
export VSOMEIP_CONFIGURATION=local_routing_test_client.json
./local_routing_test_client &
CLIENT_PID=$!

check_tcp_udp_sockets_are_open $SERIVCE_PID
sleep 2
check_tcp_udp_sockets_are_closed $CLIENT_PID

# wait that local client finishes
wait $CLIENT_PID
exit_status=$?
if [ $exit_status != 0 ] && [ $exit_status != 127 ]; then
    echo "Fail status $exit_status"
    FAIL=$FAIL+1
else
    ssh -o "StrictHostKeyChecking=no" root@172.31.1.2 "cd /data/home/qnxuser/build/test && ./external_local_routing_test_client_external_start_qnx.sh" &
fi

# Wait until client and service are finished
for job in $(jobs -p)
do
    # Fail gets incremented if either client or service exit
    # with a non-zero exit code
    wait $job
    exit_status=$?
    if [ $exit_status != 0 ] && [ $exit_status != 127 ]; then
        echo "Fail status $exit_status"
        FAIL=$FAIL+1
    fi
done

# Check if client and server both exited sucessfully and the service didnt't
# have any open 
if [ $FAIL -eq 0 ]
then
    exit 0
else
    exit 1
fi
