#!/bin/sh
# Copyright (C) 2015-2019 Bayerische Motoren Werke Aktiengesellschaft (BMW AG)
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

# Copyright Siemens, 2023

FAIL=0

if [ $# -lt 1 ]
then
    echo "Please pass a test mode to this script."
    echo "For example: $0 IN_SEQUENCE"
    echo "Valid subscription types include:"
    echo "            [IN_SEQUENCE, MIXED, INCOMPLETE, DUPLICATE, OVERLAP, OVERLAP_FRONT_BACK]"
    exit 1
fi
TESTMODE=$1
export VSOMEIP_CONFIGURATION=someip_tp_test_master.json
# start daemon
../examples/routingmanagerd/./routingmanagerd &
PID_VSOMEIPD=$!
# Start the services
./someip_tp_test_service $1 &
PID_SERIVCE=$!

sleep 1

if [ ! -z "$USE_LXC_TEST" ]; then
    echo "Waiting for 5s"
    sleep 5
    ssh -tt -i $SANDBOX_ROOT_DIR/commonapi_main/lxc-config/.ssh/mgc_lxc/rsa_key_file.pub -o StrictHostKeyChecking=no root@$LXC_TEST_SLAVE_IP "bash -ci \"set -m; cd \\\$SANDBOX_TARGET_DIR/vsomeip_lib/test; ./someip_tp_test_msg_sender 172.31.1.3 172.31.1.2 $TESTMODE\"" &
    echo "remote ssh pid: $!"
elif [ ! -z "$USE_DOCKER" ]; then
    echo "Waiting for 5s"
    sleep 5
    docker exec $DOCKER_IMAGE sh -c "cd $DOCKER_TESTS && ./someip_tp_test_msg_sender 172.31.1.3 172.31.1.2 $TESTMODE" &
else
    ssh -o "StrictHostKeyChecking=no" root@172.31.1.2 "cd /data/home/qnxuser/build/test && ./someip_tp_test_msg_sender 172.31.1.3 172.31.1.2 $TESTMODE" &
    REMOTE_SCRIPT_PID=$!
fi


# Fail gets incremented if a client exits with a non-zero exit code
echo "waiting for test service $PID_SERIVCE"
wait $PID_SERIVCE
exit_status=$?
if [ $exit_status != 0 ] && [ $exit_status != 127 ]; then
    echo "Fail status $exit_status"
    FAIL=$FAIL+1
fi

# kill the services
echo "waiting for routing manager $PID_VSOMEIPD"
kill $PID_VSOMEIPD
wait $PID_VSOMEIPD
exit_status=$?
if [ $exit_status != 0 ] && [ $exit_status != 127 ]; then
    echo "Fail status $exit_status"
    FAIL=$FAIL+1
fi
sleep 1

# Wait for the remote side to finish
echo "waiting for remote script $REMOTE_SCRIPT_PID"
wait $REMOTE_SCRIPT_PID
exit_status=$?
if [ $exit_status != 0 ] && [ $exit_status != 127 ]; then
    echo "Fail status $exit_status"
    FAIL=$FAIL+1
fi

# Check if everything went well
exit $FAIL
